﻿var glob = require("glob"),
    Promise = require('promise'),
    path = require('path'),
    fs = require('fs'),
    fileSignature = require('./file-signature.js'),
    methodKeyCreator = require('./method-key.js'),
    uuid = require('uuid'),
    //statuses = require('../statuses.js'),
    util = require('util'),
    debuglog = util.debuglog ? util.debuglog('sl') : require('../alt-debug-log.js').debuglog('sl'),
    parser = require("gitignore-parser"),
    guessMethodName = require('./guess-method-name.js').guessMethodName;

function generateNewMappingForFile(filename, fullFilename, isBranchCoverage) {
    var fileSig = {
        filename: filename,
        //status: statuses.FILE_ADDED, // "removed", "modified", "moved", "unchanged"
        methods: []
    };
    filename = filename.replace(/\\/g, '/');
    var oldContent = fs.readFileSync(fullFilename, 'utf-8');
    var sig = fileSignature(filename, oldContent, isBranchCoverage);
    if (!sig || sig.error) {
        //fileSig.status = statuses.FILE_CORRUPTED;
        return null;//fileSig;
    }
    fileSig.hash = sig.module.hash;

    var mapping = {};
    sig.methods.forEach(function (m) {
        var wholeKey = [methodKeyCreator(m.start, m.name, m.isAnonymous), m.hash, new Date().valueOf()].join('|');
        var newId = uuid(wholeKey);
        fileSig.methods.push({
            id: newId,
            isAnonymous: m.isAnonymous,
            hash: m.hash,
            sigHash: m.sigHash,
            position: m.start,
            endPosition: m.end,
            name: m.name,
            uniqueName: m.uniqueName,
            branches: m.branches
        })
    });
    return fileSig;
}

function getSlIgnore(workspacePath){
    var pattern;
    try {
        var slIgnoreFilePath = path.resolve(workspacePath, '.slignore');
        pattern = fs.readFileSync(slIgnoreFilePath, 'utf8');
        try {
            var compiled = parser.compile(pattern); //may throw if the file is invalid/corrupted
            console.log('Using .slignore file ' + slIgnoreFilePath); 
            return compiled;
        } catch(e) {
            console.error('Not using corrupted .slignore file ' + slIgnoreFilePath+". Using default behavior.");            
            throw e;
        }
    }
    catch (e) {
        // default pattern - all js files under test directory and it's sub directories.
        pattern = "test/**.js";
        return parser.compile(pattern); //should not throw again
    }    
}

function javascriptHandler(handlerParams) {
    var opts = handlerParams.buildArguments;
    var slIgnore = getSlIgnore(opts.workspacepath);
    var finalOutput = handlerParams.finalOutput;

    var p = new Promise(function (resolve, reject) {
        finalOutput.files = finalOutput.files || [];

        var files = glob.sync("**/*.js",
            {
                cwd: opts.workspacepath,
                ignore: ["node_modules/**/*.js"]
            });

        files.forEach(function (filename, fileIdx) {
            var fullFilename = path.resolve(opts.workspacepath, filename);

            // If the file is filtered by the .slignore file:
            if(slIgnore.denies(filename)) {
                console.log('Excluded [' + (fileIdx + 1) + '/' + files.length + '] ' + fullFilename);
                return; // Ignored by .slignore, skip it.
            }

            console.log('Processing [' + (fileIdx + 1) + '/' + files.length + '] ' + fullFilename);
            
            var useBranchCoverage = handlerParams.cfg.useBranchCoverage || opts.usebranchcoverage;
            var fileSig = generateNewMappingForFile(filename, fullFilename, useBranchCoverage);
            if (fileSig) finalOutput.files.push(fileSig);
        })
        resolve(true);
    });
    return p;
}

module.exports = javascriptHandler